@@ -12,11 +12,11 @@ Pg::Explain::Node - Class representing single node from query plan
 
 =head1 VERSION
 
-Version 0.61
+Version 0.62
 
 =cut
 
-our $VERSION = '0.61';
+our $VERSION = '0.62';
 
 =head1 SYNOPSIS
 
@@ -552,7 +552,6 @@ sub as_text {
     }
     my $textual = join( "\n", @lines ) . "\n";
 
-
     if ( $self->cte_order ) {
         for my $cte_name ( @{ $self->cte_order } ) {
             $textual .= $prefix_on_spaces . "CTE " . $cte_name . "\n";
@@ -585,33 +584,33 @@ First stage of anonymization - gathering of all possible strings that could and
 =cut
 
 sub anonymize_gathering {
-    my $self = shift;
+    my $self       = shift;
     my $anonymizer = shift;
 
-    if ($self->scan_on) {
+    if ( $self->scan_on ) {
         $anonymizer->add( values %{ $self->scan_on } );
     }
 
-    if ($self->extra_info) {
+    if ( $self->extra_info ) {
         for my $line ( @{ $self->extra_info } ) {
             my $copy = $line;
             next unless $copy =~ s{^((?:Join Filter|Index Cond|Recheck Cond|Hash Cond|Merge Cond|Filter|Sort Key):\s+)(.*)$}{$2};
             my $prefix = $1;
-            my $lexer = $self->_make_lexer( $copy );
+            my $lexer  = $self->_make_lexer( $copy );
             while ( my $x = $lexer->() ) {
                 next unless ref $x;
-                $anonymizer->add( $x->[1] ) if $x->[0] =~ m{\A (?: STRING_LITERAL | QUOTED_IDENTIFIER | IDENTIFIER ) \z}x;
+                $anonymizer->add( $x->[ 1 ] ) if $x->[ 0 ] =~ m{\A (?: STRING_LITERAL | QUOTED_IDENTIFIER | IDENTIFIER ) \z}x;
             }
         }
     }
 
-    for my $key (qw( sub_nodes initplans subplans ) ) {
-        next unless $self->{$key};
-        $_->anonymize_gathering( $anonymizer ) for @{ $self->{$key} };
+    for my $key ( qw( sub_nodes initplans subplans ) ) {
+        next unless $self->{ $key };
+        $_->anonymize_gathering( $anonymizer ) for @{ $self->{ $key } };
     }
 
-    if ($self->{'ctes'}) {
-        $_->anonymize_gathering( $anonymizer ) for values %{ $self->{'ctes'} };
+    if ( $self->{ 'ctes' } ) {
+        $_->anonymize_gathering( $anonymizer ) for values %{ $self->{ 'ctes' } };
     }
     return;
 }
@@ -625,8 +624,56 @@ Helper function which creates HOP::Lexer based lexer for given line of input
 sub _make_lexer {
     my $self = shift;
     my $data = shift;
+
+    ## Got from PostgreSQL 9.2devel with:
+    # SQL # with z as (
+    # SQL #     select
+    # SQL #         typname::text as a,
+    # SQL #         oid::regtype::text as b
+    # SQL #     from
+    # SQL #         pg_type
+    # SQL #     where
+    # SQL #         typrelid = 0
+    # SQL #         and typnamespace = 11
+    # SQL # ),
+    # SQL # d as (
+    # SQL #     select a from z
+    # SQL #     union
+    # SQL #     select b from z
+    # SQL # ),
+    # SQL # f as (
+    # SQL #     select distinct
+    # SQL #         regexp_replace(
+    # SQL #             regexp_replace(
+    # SQL #                 regexp_replace( a, '^_', '' ),
+    # SQL #                 E'\\[\\]$',
+    # SQL #                 ''
+    # SQL #             ),
+    # SQL #             '^"(.*)"$',
+    # SQL #             E'\\1'
+    # SQL #         ) as t
+    # SQL #     from
+    # SQL #         d
+    # SQL # )
+    # SQL # select
+    # SQL #     t
+    # SQL # from
+    # SQL #     f
+    # SQL # order by
+    # SQL #     length(t) desc,
+    # SQL #     t asc;
+
+    # Following regexp was generated by feeding list from above query to:
+    # use Regexp::List;
+    # my $q = Regexp::List->new();
+    # print = $q->list2re( @_ );
+    # It is faster than normal alternative regexp like:
+    # (?:timestamp without time zone|timestamp with time zone|time without time zone|....|xid|xml)
+    my $any_pgtype = qr{(?-xism:(?=[abcdfgilmnoprstuvx])(?:t(?:i(?:me(?:stamp(?:\ with(?:out)?\ time\ zone|tz)?|\ with(?:out)?\ time\ zone|tz)?|nterval|d)|s(?:(?:tz)?range|vector|query)|(?:xid_snapsho|ex)t|rigger)|c(?:har(?:acter(?:\ varying)?)?|i(?:dr?|rcle)|string)|d(?:ate(?:range)?|ouble\ precision)|l(?:anguage_handler|ine|seg)|re(?:g(?:proc(?:edure)?|oper(?:ator)?|c(?:onfig|lass)|dictionary|type)|fcursor|ltime|cord|al)|p(?:o(?:lygon|int)|g_node_tree|ath)|a(?:ny(?:e(?:lement|num)|(?:non)?array|range)?|bstime|clitem)|b(?:i(?:t(?:\ varying)?|gint)|o(?:ol(?:ean)?|x)|pchar|ytea)|f(?:loat[48]|dw_handler)|in(?:t(?:2(?:vector)?|4(?:range)?|8(?:range)?|e(?:r[nv]al|ger))|et)|o(?:id(?:vector)?|paque)|n(?:um(?:range|eric)|ame)|sm(?:allint|gr)|m(?:acaddr|oney)|u(?:nknown|uid)|v(?:ar(?:char|bit)|oid)|x(?:id|ml)|gtsvector))};
+
     my @input_tokens = (
         [ 'STRING_LITERAL',    qr{'(?:''|[^']+)+'} ],
+        [ 'PGTYPECAST',        qr{::"?_?$any_pgtype"?(?:\[\])?} ],
         [ 'QUOTED_IDENTIFIER', qr{"(?:""|[^"]+)+"} ],
         [ 'AND',               qr{\bAND\b}i ],
         [ 'ANY',               qr{\bANY\b}i ],
@@ -710,15 +757,15 @@ Second stage of anonymization - actual changing strings into anonymized versions
 =cut
 
 sub anonymize_substitute {
-    my $self = shift;
+    my $self       = shift;
     my $anonymizer = shift;
 
-    if ($self->scan_on) {
-        while ( my ($key, $value) = each %{ $self->scan_on } ) {
-            $self->scan_on->{$key} = $anonymizer->anonymized( $value );
+    if ( $self->scan_on ) {
+        while ( my ( $key, $value ) = each %{ $self->scan_on } ) {
+            $self->scan_on->{ $key } = $anonymizer->anonymized( $value );
         }
     }
-    if ($self->extra_info) {
+    if ( $self->extra_info ) {
         my @new_extra_info = ();
         for my $line ( @{ $self->extra_info } ) {
             unless ( $line =~ s{^((?:Join Filter|Index Cond|Recheck Cond|Hash Cond|Merge Cond|Filter|Sort Key):\s+)(.*)$}{$2} ) {
@@ -726,35 +773,38 @@ sub anonymize_substitute {
                 next;
             }
             my $output = $1;
-            my $lexer = $self->_make_lexer( $line );
+            my $lexer  = $self->_make_lexer( $line );
             while ( my $x = $lexer->() ) {
-                if (ref $x) {
-                    if ( $x->[0] eq 'STRING_LITERAL' ) {
-                        $output .= "'" . $anonymizer->anonymized( $x->[1] ) . "'";
-                    } elsif ( $x->[0] eq 'QUOTED_IDENTIFIER' ) {
-                        $output .= '"' . $anonymizer->anonymized( $x->[1] ) . '"';
-                    } elsif ( $x->[0] eq 'IDENTIFIER' ) {
-                        $output .= $anonymizer->anonymized( $x->[1] );
-                    } else {
-                        $output .= $x->[1];
+                if ( ref $x ) {
+                    if ( $x->[ 0 ] eq 'STRING_LITERAL' ) {
+                        $output .= "'" . $anonymizer->anonymized( $x->[ 1 ] ) . "'";
                     }
-                } else {
+                    elsif ( $x->[ 0 ] eq 'QUOTED_IDENTIFIER' ) {
+                        $output .= '"' . $anonymizer->anonymized( $x->[ 1 ] ) . '"';
+                    }
+                    elsif ( $x->[ 0 ] eq 'IDENTIFIER' ) {
+                        $output .= $anonymizer->anonymized( $x->[ 1 ] );
+                    }
+                    else {
+                        $output .= $x->[ 1 ];
+                    }
+                }
+                else {
                     $output .= $x;
                 }
             }
             push @new_extra_info, $output;
         }
-        $self->{'extra_info'} = \@new_extra_info;
+        $self->{ 'extra_info' } = \@new_extra_info;
     }
 
-
-    for my $key (qw( sub_nodes initplans subplans ) ) {
-        next unless $self->{$key};
-        $_->anonymize_substitute( $anonymizer ) for @{ $self->{$key} };
+    for my $key ( qw( sub_nodes initplans subplans ) ) {
+        next unless $self->{ $key };
+        $_->anonymize_substitute( $anonymizer ) for @{ $self->{ $key } };
     }
 
-    if ($self->{'ctes'}) {
-        $_->anonymize_substitute( $anonymizer ) for values %{ $self->{'ctes'} };
+    if ( $self->{ 'ctes' } ) {
+        $_->anonymize_substitute( $anonymizer ) for values %{ $self->{ 'ctes' } };
     }
     return;
 }
